@@ -1,7 +1,12 @@
-from typing import TYPE_CHECKING, Optional, Tuple, cast
+from __future__ import annotations
+
+from typing import TYPE_CHECKING, Optional, Sequence, Tuple, cast
 
 from passlib.context import CryptContext
 
+__all__ = ["PasswordManager"]
+
+
 if TYPE_CHECKING:
     from pydantic import SecretStr
 
@@ -9,12 +14,17 @@ if TYPE_CHECKING:
 class PasswordManager:
     """Thin wrapper around `passlib`."""
 
-    def __init__(self) -> None:
-        """Construct a PasswordManager."""
+    def __init__(self, hash_schemes: Sequence[str] | None = None) -> None:
+        """Construct a PasswordManager.
 
-        self.context = CryptContext(schemes=["bcrypt"], deprecated="auto")
+        Args:
+            hash_schemes: The encryption schemes to use. Defaults to ["bcrypt"].
+        """
+        if hash_schemes is None:
+            hash_schemes = ["bcrypt"]
+        self.context = CryptContext(schemes=hash_schemes, deprecated="auto")
 
-    def get_hash(self, password: "SecretStr") -> str:
+    def hash(self, password: "SecretStr") -> str:
         """Create a password hash.
 
         Args:
@@ -22,7 +32,7 @@ class PasswordManager:
         """
         return cast("str", self.context.hash(password.get_secret_value()))
 
-    def verify_and_update(self, password: "SecretStr", password_hash: str) -> Tuple[bool, Optional[str]]:
+    def verify_and_update(self, password: "SecretStr", password_hash: str) -> tuple[bool, str | None]:
         """Verify a password and rehash it if the hash is deprecated.
 
         Args:
