@@ -1,33 +1,37 @@
-from typing import TYPE_CHECKING, Any, Callable, Dict, Optional, Type
+from __future__ import annotations
+
+from typing import TYPE_CHECKING, Any, Callable
 
-from starlite_users.adapter.sqlalchemy.mixins import SQLAlchemyUserRoleMixin
-from starlite_users.adapter.sqlalchemy.repository import (
-    SQLAlchemyUserRepository,
-    SQLAlchemyUserRoleRepository,
-)
 from starlite_users.exceptions import RepositoryNotFoundException
 
+__all__ = ["get_jwt_retrieve_user_handler", "get_session_retrieve_user_handler"]
+
+
 if TYPE_CHECKING:
     from starlite import ASGIConnection
     from starlite.contrib.jwt import Token
 
     from starlite_users.adapter.sqlalchemy.mixins import RoleModelType, UserModelType
+    from starlite_users.generics import AbstractUserRepository
 
 
 def get_session_retrieve_user_handler(
-    user_model: "Type[UserModelType]", role_model: "Optional[Type[RoleModelType]]"
+    user_model: type[UserModelType],
+    role_model: type[RoleModelType],
+    user_repository_class: type[AbstractUserRepository],
 ) -> Callable:
     """Get retrieve_user_handler functions for session backends.
 
     Args:
         user_model: A subclass of a `User` ORM model.
         role_model: A subclass of a `Role` ORM model.
+        user_repository_class: A subclass of `BaseUserRepository` to use for database operations.
     """
 
     async def retrieve_user_handler(
-        session: Dict[str, Any], connection: "ASGIConnection[Any, Any, Any]"
-    ) -> "Optional[UserModelType]":
-        """Get a user from a session.
+        session: dict[str, Any], connection: ASGIConnection[Any, Any, Any]
+    ) -> UserModelType | None:
+        """Get a user from the database based on session info.
 
         Args:
             session: Starlite session.
@@ -37,16 +41,11 @@ def get_session_retrieve_user_handler(
 
         async with async_session_maker() as async_session:
             async with async_session.begin():
-                if issubclass(user_model, SQLAlchemyUserRoleMixin) and role_model:
-                    repository = SQLAlchemyUserRoleRepository(
-                        session=async_session, user_model_type=user_model, role_model_type=role_model
-                    )
-                else:
-                    repository = SQLAlchemyUserRepository(session=async_session, user_model_type=user_model)  # type: ignore[assignment]
+                repository = user_repository_class(session=async_session, user_model=user_model, role_model=role_model)
                 try:
                     user = await repository.get_user(session.get("user_id", ""))
                     if user.is_active and user.is_verified:
-                        return user  # type: ignore[return-value]
+                        return user  # type: ignore[no-any-return]
                 except RepositoryNotFoundException:
                     pass
         return None
@@ -55,17 +54,20 @@ def get_session_retrieve_user_handler(
 
 
 def get_jwt_retrieve_user_handler(
-    user_model: "Type[UserModelType]", role_model: "Optional[Type[RoleModelType]]"
+    user_model: type[UserModelType],
+    role_model: type[RoleModelType],
+    user_repository_class: type[AbstractUserRepository],
 ) -> Callable:
     """Get retrieve_user_handler functions for jwt backends.
 
     Args:
         user_model: A subclass of a `User` ORM model.
         role_model: A subclass of a `Role` ORM model.
+        user_repository_class: A subclass of `BaseUserRepository` to use for database operations.
     """
 
-    async def retrieve_user_handler(token: "Token", connection: "ASGIConnection[Any, Any, Any]") -> Optional[user_model]:  # type: ignore[valid-type]
-        """Get a user from a JWT.
+    async def retrieve_user_handler(token: Token, connection: ASGIConnection[Any, Any, Any]) -> user_model | None:  # type: ignore[valid-type]
+        """Get a user from the database based on JWT info.
 
         Args:
             token: Encoded JWT.
@@ -75,16 +77,11 @@ def get_jwt_retrieve_user_handler(
 
         async with async_session_maker() as async_session:
             async with async_session.begin():
-                if issubclass(user_model, SQLAlchemyUserRoleMixin) and role_model:
-                    repository = SQLAlchemyUserRoleRepository(
-                        session=async_session, user_model_type=user_model, role_model_type=role_model
-                    )
-                else:
-                    repository = SQLAlchemyUserRepository(session=async_session, user_model_type=user_model)  # type: ignore[assignment]
+                repository = user_repository_class(session=async_session, user_model=user_model, role_model=role_model)
                 try:
                     user = await repository.get_user(token.sub)
                     if user.is_active and user.is_verified:
-                        return user
+                        return user  # type: ignore[no-any-return]
                 except RepositoryNotFoundException:
                     pass
         return None
